/*
 * Downloaded from lenkaspace.net
 * 
 * Paper about this project: http://lenkaspace.net/papers/iteratedPrisonersDilemmaNeuralNetworks.php
 * Email: contact@lenkaspace.net
 */

package net.lenkaspace.adaptiveNavigation;

	import java.awt.FlowLayout;
import java.awt.Toolkit;

import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.Timer;
import javax.swing.UIManager;

import net.lenkaspace.adaptiveNavigation.model.Agent;
import net.lenkaspace.adaptiveNavigation.model.World;
import net.lenkaspace.adaptiveNavigation.reports.ReportController;
import net.lenkaspace.adaptiveNavigation.vo.Vector3D;


	


	@SuppressWarnings("serial")
	public class AdaptiveNagivation extends JApplet implements Runnable {
		
		private Timer painterTimer;
		private World world;
		private Painter painter;
		
		private double timeCounter; 
		private int timeUnits = 0;
		private int tryUnits = 0;
		private int tryNo = 0;
		
		
	
		private int lastPauseAt = 0;
		private int timeInterval = Settings.MAX_TIME_INTERVAL;
		
	
		
		private Agent clonedAgent;
		
		Thread myThread;
		
		/**
		 * Initialize UI: top level
		 */
		public void init() {
	        /* Turn off metal's use of bold fonts */
	        UIManager.put("swing.boldMetal", Boolean.FALSE);
	        this.getContentPane().setBounds(0, 0, Settings.WINDOW_W, Settings.WINDOW_H);
	        this.setSize(Settings.WINDOW_W, Settings.WINDOW_H);
	        //--------------- create GUI in a separate thread:
	        try {
	            javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
	                public void run() {
	                	initGUI();             	
	                }
	            });
	        } catch (Exception e) {
	            System.err.println("initGUI didn't successfully complete");
	        }    
	    }
	 
		/**
		 * Initialize UI: detailed level
		 */
	    public void initGUI() {
	    	setLayout(new FlowLayout(FlowLayout.LEFT, 10, 10));        
	        
	    	//--- create painer:
	    	painter = new Painter();
	    	add("West", painter);
	       
	    	add(new JLabel(""));
	       
	        //--- create controls panel on the right:
	        ControlsPanel cp = ControlsPanel.getSingletonObject();
	        cp.setController(this);
	        add(cp);
	        
	        //setup reportController
	        ReportController rp = ReportController.getSingletonObject();
	        rp.setController(this);
	        
	          
	    }
	    
	    /**
	     * Initialize world, time, reports
	     */
		public void initSimulation() {
	    	if (this.checkUserInput()) {	
	    		world = new World(this);
		    	timeCounter = 0;
		    	timeUnits = 0;
		    	setLastPauseAt(0);
		    	tryNo = 0;
		  
		    	this.setTimeInterval(1);
		    	ReportController.getSingletonObject().reset();

		    	painter.setWorld(world);
		    	//------ tell settings that the init is done:
		        Settings.getSingletonObject().setInitDone(true);  
		        
		        //---- start update thread: 
		        if (myThread == null) {
		        	myThread = new Thread(this);
		        	myThread.setPriority(Thread.NORM_PRIORITY);
		        	myThread.start();
		        }    
	    	}
	    }
	    
	    /**
	     * Called before initSimulation(), returns true if user input will not cause any problems
	     */
	    private boolean checkUserInput() {
			return true;
		}
	    
	    
	    //=========================================== MAIN LOOP
	    /**
	     * Update thread 
	     */
	    public void run() {
        	while (true) {
	        	if (Settings.getSingletonObject().getTimeRunning()) {
	        		ControlsPanel.getSingletonObject().clearOutput();
		        	timeCounter++;
		        	if (timeCounter == Settings.TIME_UNIT_INTERVAL) { 
		        		timeUnits++;
		        	}
		        	world.update();	
		        	if (timeCounter == Settings.TIME_UNIT_INTERVAL) {
		        		timeCounter = 0;	
		        	}
		        	ControlsPanel.getSingletonObject().showOutput(" TIME ELAPSED:"+this.timeUnits);
		   
		        	//---- end simulation and restart?
		        	int timeUnitsLimit = 160;
		        	if (tryNo == 0) {
		        		timeUnitsLimit = 160; //initial run time
		        	}
		        	if (timeUnits % timeUnitsLimit == 0 && timeUnits > 0 ) {
		        		//--- a run has finished..
		        		tryNo ++;
		        		setLastPauseAt(timeUnits);
		        		//play sound
		        		Toolkit.getDefaultToolkit().beep();
		        		ReportController.getSingletonObject().showAllReports();
		        		
		        		ControlsPanel.getSingletonObject().pauseTime();	
		        		System.err.println(" !!!!!!!!!!!!!!!!!!!!!! DONE !!!!!!!!!!!!!!!!!!!! ");
		        	} else {
		        		WaitAMoment();	
		        	}
	        	}
        	}
        }
	    
	    /**
	     * Make update thread sleep
	     */
	    protected void WaitAMoment() {
	    	try {
	    		Thread.sleep(timeInterval);
	    	} catch (InterruptedException e) { 
	    		e.printStackTrace();
	    	};
	    }
	    
	    
	    //============================================= FUNCTIONALITY
	    
	    /**
	     * Create a new agent and copy current agent's state into it
	     */
	    public void cloneAgent() {
	    	this.clonedAgent = new Agent(0,new Vector3D(300,300,0), this.world);
			this.clonedAgent.copyAgent(world.getAgent());
	    }
	    
	    /**
	     * Create a new agent, copy cloned agent's state into it and set is as current agent
	     */
	    public void replaceByClonedAgent() {
	    	if (this.clonedAgent != null) {
	    		Agent newAgent = new Agent(0,new Vector3D(300,300,0), this.world);
	    		newAgent.copyAgent(this.clonedAgent);
	    		this.world.setAgent(newAgent);
	    	}
	    }
	    
	    /**
	     * Reset time
	     */
	    public void reset() {
	    	timeUnits = 0;
	    }
	   
	    /**
	     * Starts painter update timer at a certain intervalLength and ties the time with Painter
	     * @param intervalLength
	     * 	[0..1] where 0 = slowest
	     */
	    public void startTime(double intervalLength) {
	    	if (Settings.getSingletonObject().getTimeRunning()) {
	    		painterTimer.stop();
	    	}
	    	painterTimer = new Timer(200, this.painter);
	    	painterTimer.setInitialDelay(0);
	    	painterTimer.start();	
	    }
	    
	    /**
	     * Stop painter timer
	     */
	    public void stopTime() {
	    	painterTimer.stop();
	    }	
	    
	    /**
	     * Force painter to repaint 
	     */
	    public void forceRepaint() {
	    	painter.overridePause = true;
	    	painter.repaint();
	    }
	    //=================================================================== GETTTERS / SETTERS
	    public void setTimeInterval(double intervalLength) {
	    	timeInterval = (Settings.MAX_TIME_INTERVAL - (int)((Settings.MAX_TIME_INTERVAL-Settings.MIN_TIME_INTERVAL)*intervalLength));	
	    }
	    
	    public int getTime() {
	    	return timeUnits;
	    }
	    
	    public int getTryNumber() {
	    	return tryUnits;
	    }
	    
	    public double getTimeCounter() {
	    	return timeCounter;
	    }
	    
	    public World getWorld() {
	    	return world;
	    }
	    
	   
	    
	    public int getTryNo() {
	    	return tryNo;
	    }

		public void setLastPauseAt(int lastPauseAt) {
			this.lastPauseAt = lastPauseAt;
		}

		public int getLastPauseAt() {
			return lastPauseAt;
		}
	    
	    
}
