/*
 * Downloaded from lenkaspace.net
 * 
 * Paper about this project: http://lenkaspace.net/papers/iteratedPrisonersDilemmaNeuralNetworks.php
 * Email: contact@lenkaspace.net
 */

package net.lenkaspace.adaptiveNavigation;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.lenkaspace.adaptiveNavigation.reports.ReportController;

@SuppressWarnings("serial")
public class ControlsPanel extends JPanel {
	
	private static final String REPORT_STATES = "State variables";
	private static final String REPORT_WEIGHTS = "Connections";
	private static final String REPORT_BOUNCES = "Obstacle hits";
	private static final String REPORT_GROUND_COVERED = "Ground covered";
	private static final String REPORT_TEXT = "Statistics";
	private static final String REPORT_SPEED = "Speed";
	
	private static final String CHANGE_BIG_WORLD = "Big world";
	private static final String CHANGE_SMALL_WORLD = "Small world";
	private static final String CHANGE_MAZE = "Maze";
	private static final String CHANGE_MAZE_DIAGONAL = "Diagonal maze";
	private static final String CHANGE_RANDOM = "Random";
	private static final String CHANGE_TRAINING = "Easy world";
	
	public static final String CELL_0 = "C0";
	public static final String CELL_1 = "C1";
	public static final String CELL_2 = "C2";
	public static final String CELL_3 = "C3";
	public static final String CELL_4 = "C4";
	public static final String CELL_5 = "C5";
	public static final String CELL_6 = "C6";
	public static final String CELL_7 = "C7";
	public static final String ALL = "all";
	public static final String AVERAGE = "average";
	
	public static final int OUTPUT_MICRO = 0;
	public static final int OUTPUT_MACRO = 1;
	
	
	private static ControlsPanel ref;
	//--------------------
	private AdaptiveNagivation controller;
	
	private JButton butTimeControl;
	private JButton	butSimControl;
	private JSlider sliderTimeSpeed;
	private JComboBox reportTypeCombo;
	private JComboBox reportArgumentStates;
	private JComboBox reportArgumentWeights;
	
	private JComboBox changeWorldType;
	
	private JTextPane output;
	
	private JButton resetFromCloned;
	private JButton changeWorldButton;
	
	
	
	public static ControlsPanel getSingletonObject()	  {
		  if (ref == null)
		        // it's ok, we can call this constructor
			  	ref = new ControlsPanel();		
		  return ref;
	}
	
	
	
	private ControlsPanel() {
		this.setBounds(0, 0, 400, Settings.WINDOW_H-40);
		//this.setBackground(new Color(255,255,0));
		this.setSize (400,Settings.WINDOW_H-40);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		
		
		//------------------- start and pause button:
		JPanel timeButtonsPanel = this.createFlowLayoutJPanel(30);
		
		butSimControl = new JButton();
		butSimControl.setText("START NEW SIMULATION");
		butSimControl.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
                onSimulationButtonClick();
        }	}); 
		butSimControl.setAlignmentX(Component.LEFT_ALIGNMENT);
		timeButtonsPanel.add(butSimControl);
		
		butTimeControl = new JButton();
        butTimeControl.setText(">");
        butTimeControl.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
                onTimeButtonClick();
        }	}); 
        timeButtonsPanel.add(butTimeControl);
       
		
		this.add(timeButtonsPanel);
		
		
		
		//---------------------- time settings:
		
		JPanel timeSettingsPanel = this.createFlowLayoutJPanel(60);
		timeSettingsPanel.add(new JLabel("Time speed:"));	
		
		sliderTimeSpeed = new JSlider(0, 100, 100);
		sliderTimeSpeed.setPreferredSize(new Dimension (280,40));
		sliderTimeSpeed.setMinorTickSpacing(25);
		sliderTimeSpeed.setMajorTickSpacing(25);
		sliderTimeSpeed.setPaintTicks(true);
		sliderTimeSpeed.setPaintLabels(true);
		sliderTimeSpeed.addChangeListener( new ChangeListener() {
        	public void stateChanged(ChangeEvent e) {
        		onTimeSpeedChange();
        	}
        }); 
		timeSettingsPanel.add(sliderTimeSpeed);
		
		this.add(timeSettingsPanel);
		this.add(this.createDivider(0));
		
		setTimeControlsEnabled(false); 
		
		 
		//-------output:
		output = new JTextPane();	
		output.setPreferredSize(new Dimension(0, 350));
		//output.setSize(new Dimension(50, 350));
		/*scroll1 = new JScrollPane(output);
		scroll1.setPreferredSize(new Dimension(380, 180));
		scroll1.setAlignmentX(Component.LEFT_ALIGNMENT); */
		this.add(output);
		this.add(this.createDivider(0));
		this.add(this.createSpace(10));
		 
		//-------------------- reports:
		JPanel reportsPanel = this.createFlowLayoutJPanel(30);
		reportsPanel.add(new JLabel("Report:"));
		
		//----- type combo box:
        String[] reportTypes = { REPORT_STATES, REPORT_WEIGHTS, REPORT_BOUNCES, REPORT_GROUND_COVERED, REPORT_SPEED, REPORT_TEXT };
        reportTypeCombo = new JComboBox(reportTypes);
        reportTypeCombo.setSelectedIndex(0);
        reportsPanel.add(reportTypeCombo);
        reportTypeCombo.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
            onReportTypeSelected();
        }	});
        
        String[] reportArgs1 = { ControlsPanel.CELL_0,ControlsPanel.CELL_1,ControlsPanel.CELL_2,ControlsPanel.CELL_3,ControlsPanel.CELL_4,ControlsPanel.CELL_5,ControlsPanel.CELL_6,ControlsPanel.CELL_7,ControlsPanel.ALL,ControlsPanel.AVERAGE };
        reportArgumentStates = new JComboBox(reportArgs1);
        reportArgumentStates.setSelectedIndex(8);
        reportsPanel.add(reportArgumentStates);
        
        String[] reportArgs2 = { ControlsPanel.CELL_0,ControlsPanel.CELL_1,ControlsPanel.CELL_2,ControlsPanel.CELL_3,ControlsPanel.CELL_4,ControlsPanel.CELL_5,ControlsPanel.CELL_6,ControlsPanel.CELL_7,ControlsPanel.ALL };
        reportArgumentWeights = new JComboBox(reportArgs2);
        reportArgumentWeights.setSelectedIndex(8);
        reportsPanel.add(reportArgumentWeights);
        
        // check if anything needs to be disabled:
        this.onReportTypeSelected();
        
        //--- OPEN button
        JButton getReportButton = new JButton();
        getReportButton.setText("Open");
        getReportButton.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
                onOpenReportClick();
        }	});
        reportsPanel.add(getReportButton); 
		
        this.add(reportsPanel);
        this.add(this.createDivider(10));
	
        
        //------------ draw path button:
        JPanel drawPathPanel = this.createFlowLayoutJPanel(30);
        JButton drawPathButton = new JButton();
        drawPathButton.setText("Draw path");
        drawPathButton.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
                onDrawPathClick();
        }	});
        drawPathPanel.add(drawPathButton);
        
        
        
        this.add(drawPathPanel);
        
        //-----
        JPanel clonePanel = this.createFlowLayoutJPanel(30);
        
        JButton cloneButton = new JButton();
        cloneButton.setText("Clone");
        cloneButton.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
                onCloneClick();
        }	});
        clonePanel.add(cloneButton);
        
        JButton resetPositionButton = new JButton();
        resetPositionButton.setText("Reset");
        resetPositionButton.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
                onResetPositionClick();
        }	});
        clonePanel.add(resetPositionButton);
        
        resetFromCloned = new JButton();
        resetFromCloned.setText("Reset from cloned");
        resetFromCloned.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
                onResetPositionFromClonedClick();
        }	});
        resetFromCloned.setEnabled(false);
        clonePanel.add(resetFromCloned);
        
        this.add(clonePanel);
       
        
        //------------ change world:
        JPanel changeWorldPanel = this.createFlowLayoutJPanel(30);
        
        changeWorldPanel.add(new JLabel("Change world to:"));
        
        String[] changeTypes = { CHANGE_BIG_WORLD, CHANGE_SMALL_WORLD, CHANGE_TRAINING, CHANGE_MAZE, CHANGE_MAZE_DIAGONAL, CHANGE_RANDOM };
        changeWorldType = new JComboBox(changeTypes);
        changeWorldPanel.add(changeWorldType);
        
        changeWorldButton = new JButton();
        changeWorldButton.setText("Change");
        changeWorldButton.addActionListener(new ActionListener() {	public void actionPerformed(ActionEvent e) {
                onChangeWorldClick();
        }	});
        changeWorldPanel.add(changeWorldButton);
        
        
        this.add(changeWorldPanel);  
        this.add(this.createDivider(10));
        
	}
	
	
	public void setController(AdaptiveNagivation controller_) {
		this.controller = controller_;
	}
	
	


	//=========================================================== ACTIONS
	
	public void clearOutput() {
		output.setText("");
	}
	public void showOutput(String text) {
		output.setText(output.getText() + text +  "\n");
	}
	
	public void pauseTime() {
		if (Settings.getSingletonObject().getTimeRunning()) {
			this.onTimeButtonClick();
		}
	}
	
	public void restartSimulation() {
		this.onSimulationButtonClick();
	}
	
	
	//-------------------------------------- single buttons controls
	
	private void onDrawPathClick() {
		Settings.getSingletonObject().setDrawPath(true);
		this.controller.forceRepaint();
		
	}
	
	private void onCloneClick() {
		this.controller.cloneAgent();
		resetFromCloned.setEnabled(true);
	}
	
	private void onResetPositionClick() {
		this.controller.reset();
		ReportController.getSingletonObject().reset();
		ReportController.getSingletonObject().clearAgentPositions();
		this.controller.getWorld().getAgent().reset();
		this.controller.forceRepaint();
		
	}
	
	private void onResetPositionFromClonedClick() {
		this.controller.reset();
		ReportController.getSingletonObject().reset();
		ReportController.getSingletonObject().clearAgentPositions();
		this.controller.replaceByClonedAgent();
		this.controller.forceRepaint();
		
	}
	
	private void onChangeWorldClick() {
		ReportController.getSingletonObject().clearAgentPositions();
		
		String selectedType = (String) this.changeWorldType.getSelectedItem();
		if (selectedType == CHANGE_BIG_WORLD) {
			this.controller.getWorld().createBigWorld();
		} else if (selectedType == CHANGE_SMALL_WORLD) {
			this.controller.getWorld().createSmallWorld();
		} else if (selectedType == CHANGE_MAZE) {
			this.controller.getWorld().createMaze();
		} else if (selectedType == CHANGE_MAZE_DIAGONAL) {
			this.controller.getWorld().createDiagonalMaze();
		} else if (selectedType == CHANGE_RANDOM) {
			this.controller.getWorld().createRandomWorld();
		} else if (selectedType == CHANGE_TRAINING) {
			this.controller.getWorld().createTrainingWorld();
		} 
		
		this.controller.forceRepaint();
	}

	
	//-------------------------------------- report controls
	private void onReportTypeSelected() {
		String selectedType = (String) this.reportTypeCombo.getSelectedItem();
		if (selectedType == REPORT_BOUNCES || selectedType == REPORT_GROUND_COVERED || selectedType == REPORT_SPEED) {
			this.reportArgumentStates.setEnabled(false);
			this.reportArgumentWeights.setEnabled(false);
		} else {
			this.reportArgumentStates.setEnabled(true);
			this.reportArgumentWeights.setEnabled(true);
			if (selectedType == REPORT_WEIGHTS) {
				this.reportArgumentStates.setVisible(false);
				this.reportArgumentWeights.setVisible(true);
			} else if (selectedType == REPORT_STATES) {
				this.reportArgumentStates.setVisible(true);
				this.reportArgumentWeights.setVisible(false);
			} 
		}
	}
	private void onOpenReportClick() {
		String selectedType = (String) this.reportTypeCombo.getSelectedItem();
		if (selectedType == REPORT_STATES) {
			String selectedArgument = (String) this.reportArgumentStates.getSelectedItem();
			ReportController.getSingletonObject().showReport(ReportController.REPORT_STATES,this.translateArgument(selectedArgument));	
		} else if (selectedType == REPORT_WEIGHTS) {
			String selectedArgument = (String) this.reportArgumentWeights.getSelectedItem();
			ReportController.getSingletonObject().showReport(ReportController.REPORT_WEIGHTS,this.translateArgument(selectedArgument));	
		} else if (selectedType == REPORT_BOUNCES) {
			ReportController.getSingletonObject().showReport(ReportController.REPORT_BOUNCES,-1);	
		} else if (selectedType == REPORT_GROUND_COVERED) {
			ReportController.getSingletonObject().showReport(ReportController.REPORT_GROUND_COVERED,-1);	
		}  else if (selectedType == REPORT_TEXT) {
			ReportController.getSingletonObject().showReport(ReportController.REPORT_TEXT,-1);	
		}  else if (selectedType == REPORT_SPEED) {
			ReportController.getSingletonObject().showReport(ReportController.REPORT_SPEED,-1);	
		} 
	}	
	
	

	//------------------------------------ simulation controls
	
	
	private void onSimulationButtonClick() {
		Settings settings = Settings.getSingletonObject();	
		controller.initSimulation();
		setTimeControlsEnabled(true);
		if (!settings.getTimeRunning()) {
			onTimeButtonClick();
		}
		resetFromCloned.setEnabled(false);
	}
	
	//-------------------------------------- time controls
	private void onTimeButtonClick() {
		Settings settings = Settings.getSingletonObject();
		
		if (settings.getTimeRunning()) {
			//---- pause clicked:
			controller.stopTime();
			butTimeControl.setText(">");
			settings.setTimeRunning(false);
			
			//-- enable the Change button:
			changeWorldButton.setEnabled(true);
			
		} else {
			//----- play clicked:	
			controller.startTime((double)sliderTimeSpeed.getValue() / 100);
			butTimeControl.setText("||");
			settings.setTimeRunning(true);
			
			//-- disable the Change button:
			changeWorldButton.setEnabled(false);
		}
		
	}
	
	private void onTimeSpeedChange() {
		if (Settings.getSingletonObject().getTimeRunning()) {
			controller.setTimeInterval((double)sliderTimeSpeed.getValue()/100);
			//controller.startTime((double)sliderTimeSpeed.getValue()/100);
		}
	}
	
	private void setTimeControlsEnabled(boolean enabled_) {
		sliderTimeSpeed.setEnabled(enabled_);
		butTimeControl.setEnabled(enabled_);
	}
	
	//======================================== COMPONENTS
	
	private JPanel createFlowLayoutJPanel(int height) {
		JPanel panel = new JPanel();
		Dimension panelSize = new Dimension (this.getSize().width, height);
		panel.setSize(panelSize);
		panel.setPreferredSize(panelSize);
		panel.setAlignmentX(Component.LEFT_ALIGNMENT);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
		return panel;
	}
	
	private JPanel createDivider(int spaceAround) {
		JPanel panel = this.createFlowLayoutJPanel(spaceAround*2 + 1);
		panel.add(this.createSpace(spaceAround));
		
		
		JPanel dividerPanel = new JPanel();
		Dimension panelSize = new Dimension (this.getSize().width, 1);
		
		dividerPanel.setSize(panelSize);
		dividerPanel.setPreferredSize(panelSize);
		dividerPanel.setAlignmentX(Component.LEFT_ALIGNMENT - 10);
		dividerPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 10));
		dividerPanel.setBackground(Color.LIGHT_GRAY);
		panel.add(dividerPanel);
		
		panel.add(this.createSpace(spaceAround));
		return panel;
	}
	
	private JPanel createSpace(int height) {
		JPanel panel = new JPanel();
		Dimension panelSize = new Dimension (this.getSize().width, height);
		panel.setSize(panelSize);
		panel.setPreferredSize(panelSize);
		panel.setAlignmentX(Component.LEFT_ALIGNMENT - 10);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 10));
		return panel;
	}
	
	@SuppressWarnings("unused")
	private JTextField createTextField(String defaultVal) {
		JTextField textField = new JTextField(5);
		textField.setAlignmentX(Component.LEFT_ALIGNMENT);
		textField.setPreferredSize(new Dimension(50,30));
		textField.setText(defaultVal);
		return textField;
	}
	
	//------------------------
	private int translateArgument(String argument) {
		if (argument == ControlsPanel.CELL_0) {
			return ReportController.CELL_0;
		} else if (argument == ControlsPanel.CELL_1) {
			return ReportController.CELL_1;
		} else if (argument == ControlsPanel.CELL_2) {
			return ReportController.CELL_2;
		} else if (argument == ControlsPanel.CELL_3) {
			return ReportController.CELL_3;
		} else if (argument == ControlsPanel.CELL_4) {
			return ReportController.CELL_4;
		} else if (argument == ControlsPanel.CELL_5) {
			return ReportController.CELL_5;
		} else if (argument == ControlsPanel.CELL_6) {
			return ReportController.CELL_6;
		} else if (argument == ControlsPanel.CELL_7) {
			return ReportController.CELL_7;
		} else if (argument == ControlsPanel.ALL) {
			return ReportController.ALL;
		} else if (argument == ControlsPanel.AVERAGE) {
			return ReportController.AVERAGE;
		} 
		
		return -1;
	}
	
	
	
}

