package net.lenkaspace.adaptiveNavigation;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.net.URL;

import javax.imageio.ImageIO;

import net.lenkaspace.adaptiveNavigation.model.Agent;
import net.lenkaspace.adaptiveNavigation.model.Cell;
import net.lenkaspace.adaptiveNavigation.model.Food;
import net.lenkaspace.adaptiveNavigation.model.Obstacle;
import net.lenkaspace.adaptiveNavigation.model.World;
import net.lenkaspace.adaptiveNavigation.reports.ReportController;
import net.lenkaspace.adaptiveNavigation.vo.Vector3D;


@SuppressWarnings("serial")
class Painter extends Component implements ActionListener  {
    

    protected BufferedImage imgAgent = null;
    protected BufferedImage imgCell = null;
    protected BufferedImage imgObstacle = null;
    protected BufferedImage imgFood = null;
    protected BufferedImage imgRedDot = null;
    protected BufferedImage imgGrayDot = null;
    protected World world;
   
   
    public boolean overridePause = false;
    
    protected Graphics graphics;
    
    
    public Painter() {
    	imgAgent = loadImage("agent");
    	imgCell = loadImage("cell");
    	imgObstacle = loadImage("obstacle");
    	imgFood = loadImage("food");
    	imgRedDot = loadImage ("redDot");
    	imgGrayDot = loadImage("grayDot");
    }
    
    public void setWorld(World world_) {
    	world = world_;
    }
    
    
    public void actionPerformed(ActionEvent e) {
    	this.repaint();
		
	}
    
   
    
   
    
    
    
    
    public void paint(Graphics g) {
    	
    	
	    Dimension size = getSize();
	    
	    Settings settings = Settings.getSingletonObject();
	    if (settings.getInitDone() && (settings.getTimeRunning() || overridePause)) {
		    if (overridePause) {
		    	//make sure only paints once..
		    	overridePause = false;
		    }
	    	//---------------- draw agent:
		    Agent agent = world.getAgent();
		   // for (Agent agent : world.getAgents()) {;
		    	drawImageAt(imgAgent,agent.getWorldPosition(),agent.getRotationRads(), g);
		    	//----------------------- draw cells:
		    	for (Cell cell : agent.getCells()) {
		    		if (cell.id  < 9 ) {
		    			
		    			if (cell.getIsSensing() ) {
		    				
		    				Vector3D cellPosition = cell.getWorldPosition();
			    			drawImageAt(imgCell, cellPosition, cell.getWorldRotationRads(), g);
		    					g.drawString(String.valueOf(cell.id), (int)cellPosition.x, (int)cellPosition.y);
		    					//g.drawString(String.valueOf(relVec.y), (int)cellPosition.x-20, (int)cellPosition.y+10);
		    				
		    			}
		    		}
		    		
		    	}
		   // }
		    
		    //---------------- draw obstacles:
		    for (Obstacle obstacle : world.getObstacles()) {;
		    	drawImageAt(imgObstacle,obstacle.getPosition(),obstacle.getRotationRads(), g);
		    	//g.drawString(String.valueOf(obstacle.id), (int)obstacle.getWorldPosition().x, (int)obstacle.getWorldPosition().y  -20);
		    }
		    
		    //---------------- draw food:
		    for (Food food : world.getFood()) {
		    	drawImageAt(imgFood,food.getPosition(),food.getRotationRads(), g);
		    }
		   
		    //-------- draw path:
		    int[][] groundRecord = ReportController.getSingletonObject().getGroundRecord();
		    if (Settings.getSingletonObject().getDrawPath()) {
		    	Settings.getSingletonObject().setDrawPath(false);
		    	for (int x = 0; x<Settings.WORLD_W; x++) {
		    		for (int y=0; y< Settings.WORLD_H; y++) {
		    			if (groundRecord[x][y] == ReportController.GROUNDMARK_AGENT_ORIGIN) {
		    				drawImageAt(imgGrayDot,new Vector3D(x,y,0),0, g);
		    			} else if (groundRecord[x][y] == ReportController.GROUNDMARK_AGENT_ORIGIN_BUMP) {
		    				drawImageAt(imgRedDot,new Vector3D(x,y,0),0, g);
		    			}
		    		}
		    	}
		    	
		    	// pause the simulation:
				ControlsPanel.getSingletonObject().pauseTime();
		    }
		    
		
	    }
	    
	   

 	    //-------- draw border
 	    g.setColor(new Color(0,0,0 ));   
 	    g.drawRect(0, 0, size.width-1, size.height-2); 
 
    }
    
    //================================================ IMAGES =============
    
    public void drawImageAt(BufferedImage img_, Vector3D pos_, Graphics g) {
    	int imgX = (int) (pos_.x - img_.getWidth(null)/2);
    	int imgY = (int) (pos_.y - img_.getHeight(null)/2);
        g.drawImage(  img_, imgX, imgY, img_.getWidth(null), img_.getHeight(null),null);  
    }
    
    public void drawImageAt(BufferedImage img_, Vector3D pos_, double rot_, Graphics g ) {
    	Graphics2D g2d = (Graphics2D)g;
        AffineTransform origXform = g2d.getTransform();
        //--- rotate about image's origin:
        g2d.rotate(rot_, pos_.x , pos_.y );
        //-- adjust x and y based on image size:
        int imgX = (int) (pos_.x - img_.getWidth(null)/2);
    	int imgY = (int) (pos_.y - img_.getHeight(null)/2);
        g2d.drawImage(img_, imgX, imgY, this);
        //---- prevent the transformation from affecting other images:
        g2d.setTransform(origXform);	
    }
   
    
    private BufferedImage loadImage(String name) {
        String imgFileName = "images/"+name+".png";
        URL url =  AdaptiveNagivation.class.getResource(imgFileName);
        BufferedImage img = null;
        try {
            img =  ImageIO.read(url);
        } catch (Exception e) {
        }
        return img;
    }
    
    //===================================================================================
    public Dimension getPreferredSize(){
        return new Dimension(Settings.WORLD_W, Settings.WORLD_H);
    } 



	
}