package net.lenkaspace.adaptiveNavigation.model;

import net.lenkaspace.adaptiveNavigation.Settings;
import net.lenkaspace.adaptiveNavigation.vo.Vector3D;



public class BaseSituatedModel extends BaseModel {
	
	
	protected double rotation;
	protected Vector3D position;
	protected Vector3D size;
	protected World world;
	protected int speed;
	
	protected int updateCounter;
	
	/*
	 * Steering force: 	y-component (SPEED): [0..1] where 1 is maximum speed 
	 * 					x-component (ROTATION): [-1..1] where 1 is 180deg
	 */
	protected Vector3D steeringForce;

	
	public BaseSituatedModel(int id_, Vector3D pos_, Vector3D size_, World world_) {
		super(id_);
		world = world_;
		position = pos_;
		size = size_;
		steeringForce = new Vector3D(0,0,0);
		speed = 2;
		updateCounter = 0;
		
	}
	

	public void update() {	
		super.update();
		applyThrusters();
	}
	
	public void applyThrusters()  {	
		if (this.getClass() == Agent.class) {
			//ControlsPanel.getSingletonObject().showOutput("APPLY THRUST");
		}
		//---------------------------------- normalize the force components:
		normalizeSteeringForce();	
    	//---------------------------------- calculate the movement:
		double xVelocity = Math.sin(getRotationRads()) * steeringForce.y * this.speed;
    	double yVelocity = -Math.cos(getRotationRads()) * steeringForce.y * this.speed;
    	
    	position.x += xVelocity;
    	position.y += yVelocity; 
    	
    	BaseSituatedModel collidedObject = null;
    	//---------------------------------- adjust position based on objects it collides with:
    	if (this.getClass() == Agent.class) {
	    	for (Obstacle obstacle : world.getObstacles()) {
				if (obstacle != this) {
					double distanceOfMiddles = Math.hypot(position.x - obstacle.getWorldPosition().x, position.y - obstacle.getWorldPosition().y);
			    	if (distanceOfMiddles < (size.x /2 + obstacle.getSize().x/2)) {
			        	position.x -= xVelocity;
			        	position.y -= yVelocity; 
			        	collidedObject = obstacle;
			        	break;	
			    	}
				}
			}
    	}
    	
    	
    	/*if (obstacleId >= 0) {
    		Obstacle obstacle = this.world.getObstacleWithId(obstacleId);
    		Vector3D v = this.getRelativeVectorTo2(obstacle);
    		double goBackBy = - (this.getSize().x/2 + obstacle.getSize().x / 2 - 1 *v.y );// (50.00 - v.y) ;// (this.getSize().x + obstacle.getSize().x + v.y);
   		 	double xv = Math.sin(getRotationRads()) * steeringForce.y * goBackBy;
   		 	double yv = -Math.cos(getRotationRads()) * steeringForce.y * goBackBy;
   		 	position.x += ( xv);
   		 	position.y += ( yv);
    		
    	}*/
    	
    	this.handleObjectCollisions(collidedObject);
    	
    	//---------------------------------- adjust position in the borderless world:
    	adjustBorderlessWorldCoordins();	
	}
	
	protected void handleObjectCollisions(BaseSituatedModel collidedObject) {
		
	}

	
	//================================================== ADJUSTMENTS
	/*
	 * adjustBorderlessWorldCoordins
	 * 		- sends agent to the left side when went away through right, etc.
	 */
	
	public void adjustBorderlessWorldCoordins() {
		if (position.x - size.x/4 > Settings.WORLD_W ) {
			position.x = - size.x / 4;
			//this.getSteeringForce().x = -this.getSteeringForce().x;
		} else if (position.x + size.x/4 < 0 ) {
			position.x = Settings.WORLD_W + size.x / 4;
		}
		
		if (position.y - size.y/4 > Settings.WORLD_H ) {
			position.y = - size.y / 4;
		} else if (position.y + size.y /4   < 0 ) {
			position.y = Settings.WORLD_H  + size.y / 4;
		}
	}
	
	
	public void turnByAngle(double angle_) {
		steeringForce.x += angle_/180;
	}
	
	public void normalizeSteeringForce() {
		if (steeringForce.y > 1){
			steeringForce.y = 1;
		} else if (steeringForce.y < -1){
			steeringForce.y = -1;
		}
		
		if (steeringForce.x > 1){
			steeringForce.x = -1 + (steeringForce.x - 1); // reached 180, set to -180
		} else if (steeringForce.x < -1){
			steeringForce.x = 1 + (steeringForce.x + 1); // reached -180, set to 180
		} 
	}
	
	
	//=========================================== VECTORS
	
	protected Vector3D getOrientationVector() {
		Vector3D v = new Vector3D(this.steeringForce.x, this.steeringForce.y,0);
		v.normalize();
		return v;
	}
	
	protected Vector3D getLookAtVector() {
		Vector3D v = this.getOrientationVector();
		v.multiplyBy(5);
		return v;
	}


	protected BaseSituatedModel getClosestObject() {
		double minDistance = 99999;
		int closestObjectId = -1;
		boolean foundInAgents = false;
		//go through all obstacles in the world:
		for (Obstacle obstacle : world.getObstacles()) {
			if (obstacle != this) {
				// get vector to an obstacle:
				Vector3D relVec = this.getRelativeVectorTo(obstacle);
				if (relVec != null && this.checkObjectInRange(relVec)) { // if is sensing at all
					//only fires for the closest obstacle, find the closes one:
					if (Math.abs(relVec.y) < minDistance) {
						minDistance = Math.abs(relVec.y);
						closestObjectId = obstacle.id;
					}
				}
			}
		}
		
		//go through all agents in the world:
		/*for (Agent agent : world.getAgents()) {
			if (agent != this) {
				// get vector to an aget:
				Vector3D relVec = this.getRelativeVectorTo(agent);
				if (relVec != null) { // if is sensing at all
					//only fires for the closest obstacle, find the closes one:
					if (Math.abs(relVec.y) < minDistance) {
						minDistance = Math.abs(relVec.y);
						foundInAgents = true;
						closestObjectId = agent.id;
					}
				}
			}
		}*/
		
		//get the object that was found closest:
		if (closestObjectId >=0 ) {
			if (foundInAgents) {
				//Agent agent = world.getAgentWithId(closestObjectId);
				return world.getAgent();
			} else {
				Obstacle obstacle = world.getObstacleWithId(closestObjectId);
				return obstacle;
			}
			
			
		}
		
		return null;
	}
	protected Vector3D getVectorToClosesObject() {
		BaseSituatedModel object = this.getClosestObject();
		if (object != null) {
			return(this.getRelativeVectorTo(object));
		}
		return null;
	}
	
	/*
	 * checkObjectInRange
	 * 	- returns true by default (object has infinite range), but can be overwritten by subclasses to specify range
	 * 	@param vecToObject vector to tested object
	 */
	protected boolean checkObjectInRange(Vector3D vecToObject) {
		return true;
	}
	
	
	protected Vector3D getRelativeVectorTo(BaseSituatedModel object_) {
		
		Vector3D tempVector = Vector3D.subtractVectors(this.getWorldPosition(), object_.getWorldPosition());
		Vector3D u = this.rotate2D(this.getWorldRotationRads(), tempVector);
		u.normalize();
		
		//distance from radial border of cell to radial border of obstacle
		/*u.y = u.y * Math.hypot(	(this.getWorldPosition().x- object_.getWorldPosition().x)-(this.getSize().x/2 + object_.getSize().x/2),
								(this.getWorldPosition().y- object_.getWorldPosition().y) -(this.getSize().y/2 + object_.getSize().y/2));
								*/
		double distance = Math.hypot(	(this.getWorldPosition().x- object_.getWorldPosition().x),
								(this.getWorldPosition().y- object_.getWorldPosition().y));
		distance -= (this.getSize().x/2 + object_.getSize().x/2); //!! only true for objects with width=height
		//add direction:
		if (u.y < 0) {
			distance = -distance;
		}
		u.y = distance;
		//clip the distance with a certain precision:
		//u.x = this.roundNumber(u.x);
		//u.y = this.roundNumber(distance);
		
		return u;
	}
	

	
	
	protected Vector3D rotate2D(double orientation_, Vector3D vector_) {
		Vector3D tempVec = new Vector3D();
		tempVec.x = vector_.x*Math.cos(orientation_) + vector_.y*Math.sin(orientation_);
		tempVec.y = - vector_.x*Math.sin(orientation_) + vector_.y*Math.cos(orientation_);
		return tempVec;
	}
	
	
	
	//================================================== GETTERS / SETTERS
	
	/*
	 * getRotationRads()
	 * 		- returns angle in radians
	 */
	public double getRotationRads() {
		//return Math.toRadians(steeringForce.x*180);
		return Math.toRadians(this.getSteeringForce().x*180);
	}
	
	/*
	 * getRotationRads()
	 * 		- returns angle in radians
	 */
	public double getWorldRotationRads() {
		//return Math.toRadians(steeringForce.x*180);
		return Math.toRadians(this.getWorldSteeringForce().x*180);
	}
	
	public Vector3D getPosition() {
		return position;
	}
	
	public void setPosition(Vector3D newPos) {
		this.position = newPos;
	}
	
	/*
	 * getWorldPosition()
	 * 	- used for painter, should be overriden by subclasses if their position is relative to other BaseSituatedModels
	 */
	public Vector3D getWorldPosition() {
		return position;
	}
	
	/*
	 * getWorldPosition()
	 * 	- used for painter, should be overriden by subclasses if their position is relative to other BaseSituatedModels
	 */
	public Vector3D getWorldSteeringForce() {
		return steeringForce;
	}
	
	public Vector3D getSize() {
		return size;
	}
	
	public Vector3D getSteeringForce() {
		return steeringForce;
	}
	
	public int getSpeed() {
		return speed;
	}
	
	
}
