/*
 * Downloaded from lenkaspace.net
 * 
 * Paper about this project: http://lenkaspace.net/papers/iteratedPrisonersDilemmaNeuralNetworks.php
 * Email: contact@lenkaspace.net
 */

package net.lenkaspace.adaptiveNavigation.model;

import java.util.ArrayList;
import java.util.List;

import net.lenkaspace.adaptiveNavigation.AdaptiveNagivation;
import net.lenkaspace.adaptiveNavigation.Maths;
import net.lenkaspace.adaptiveNavigation.Settings;
import net.lenkaspace.adaptiveNavigation.reports.ReportController;
import net.lenkaspace.adaptiveNavigation.vo.Vector3D;

/*
 * World class
 * - contains agents and implements the genetic algorithm
 */
public class World  {
	//---- constats
	public static int NUM_AGENTS = 200; //must be even number

		
	//-------------
	private Agent agent;
	private ArrayList <Obstacle> obstacles;
	private ArrayList <Food> food;
	private AdaptiveNagivation owner;
		
	public World(AdaptiveNagivation owner_) { 
		owner = owner_;		
		agent = new Agent(0, new Vector3D (300,300,0), this);	
		obstacles = new ArrayList <Obstacle> ();
		food = new ArrayList <Food> ();	
		this.createMaze();
	}
	

	public void update() {
		//update objects:
		agent.update();
			
		//update obstacles:
		for (Obstacle o : obstacles) {
			o.update();
		}		
	}
	
	//=================================================================================== CREATION
	public void createTrainingWorld() {
		this.createBigWorld();
		this.createObstacleMesh(150, 150, 200, 15);
		this.createObstacleMesh(200, 450, 300, 15);
		this.createObstacleMesh(50, 200, 15, 100);
		this.createObstacleMesh(450, 50, 15, 250);	
	}
	
	public void createRandomWorld() {
		
		//add big bounds:
		this.createBigWorld();
		
		//add random obstacles:
		for (int i = 0; i<8; i++) {
			int startX = Maths.getSingletonObject().getRandomInteger(0, Settings.WORLD_W-100);
			int startY = Maths.getSingletonObject().getRandomInteger(0, Settings.WORLD_H-100);
			int width = 0;
			int height = 0;
			if (i<4) {
				height = Maths.getSingletonObject().getRandomInteger(50,500);
			} else {
				height = Maths.getSingletonObject().getRandomInteger(0,40);
			}
			if (i>=4) {
				width = Maths.getSingletonObject().getRandomInteger(50,500);
			} else {
				width = Maths.getSingletonObject().getRandomInteger(0,40);
			}
			this.createObstacleMesh(startX, startY, width, height);
		}
		
		for (int i = 0; i<5; i++) {
			int startX = Maths.getSingletonObject().getRandomInteger(0, Settings.WORLD_W);
			int startY = Maths.getSingletonObject().getRandomInteger(0, Settings.WORLD_H);
			double steepness = Maths.getSingletonObject().getRandomDouble(false)*2 - 1;
			int targetLength = Maths.getSingletonObject().getRandomInteger(5, 15);
			this.createObstacleDiagonal(startX, startY, steepness, targetLength);
		}
	}
	
	public void createMaze() {
		//remove all :
		this.removeAllObstacles();
		//add big bounds:
		this.createBigWorld();
		// maze:
		this.createObstacleMesh(194, 120, 15, 250); //I vertical
		this.createObstacleMesh(150, 370, 100, 15); //I vertical bottom
		this.createObstacleMesh(390, 90, 15, 400); //L verical
		this.createObstacleMesh(390, 440, 100, 15); //L bottom
		this.createObstacleMesh(108, 144, 200, 15); //T top
		this.createObstacleMesh(7, 302, 48, 15); // small left middle
		this.createObstacleMesh(138, 504, 240, 15); //bottom
		this.createObstacleMesh(Settings.WORLD_W-60, 0, 60, 100); //big top right	
	}
	
	public void createDiagonalMaze() {
		//remove all :
		this.removeAllObstacles();
		//add big bounds:
		this.createBigWorld();
		// maze:
		this.createObstacleDiagonal(100,0,-0.2,10);
		this.createObstacleDiagonal(470,240,-0.5,7); //R middle diagonal
		this.createObstacleDiagonal(200,0,1,10); //top middle diagnoal L
		this.createObstacleDiagonal(450,0,-0.8,10); //top meddle diagnoal R
		this.createObstacleMesh(380, 100, 260, 15); //top L horizontal
		this.createObstacleMesh(230, 140, 100, 15);
		this.createObstacleMesh(0, 400, 200, 15);
		this.createObstacleMesh(430, 340, 15, 170); //bottom vertical
		this.createObstacleDiagonal(190,400,0.5,6); //adj diagnoal
		this.createObstacleDiagonal(170,300,-0.7,14); //adj diagnoal
		this.createObstacleDiagonal(145,250,0.5,8); //adj  from top of previous diagnoal
		this.createObstacleMesh(280, 570, 50, 30); //bottom middle
		this.createObstacleMesh(550, 400, 100, 230); //bottom L
		
		this.createObstacleMesh(400, 240, 100, 15); //middle horizontal
		this.createObstacleMesh(320, 360, 120, 15); //middle horizontal	
	}
	
	public void createBigWorld() {
		//remove all :
		this.removeAllObstacles();
		//---
		this.createObstacleMesh(0, 0, Settings.WORLD_W, Obstacle.BODY_H);
		this.createObstacleMesh(0, 0, Obstacle.BODY_W, Settings.WORLD_H);
		this.createObstacleMesh(Settings.WORLD_W, 0, Obstacle.BODY_W, Settings.WORLD_H);
		this.createObstacleMesh(0, Settings.WORLD_H, Settings.WORLD_W+Obstacle.BODY_W, Obstacle.BODY_W); 
	}
	
	public void createSmallWorld() {
		//remove all :
		this.removeAllObstacles();
		//---
		this.createObstacleMesh(-Obstacle.BODY_W/2, 170, Settings.WORLD_W+Obstacle.BODY_W, Obstacle.BODY_H);
		this.createObstacleMesh(-Obstacle.BODY_W/2, 400, Settings.WORLD_W+Obstacle.BODY_W, Obstacle.BODY_H);
		this.createObstacleMesh(100, -Obstacle.BODY_H/2, Obstacle.BODY_W, Settings.WORLD_H);
		this.createObstacleMesh(400, -Obstacle.BODY_H/2, Obstacle.BODY_W, Settings.WORLD_H);
	}
	
	public void createObstacleMesh(int startingX, int startingY, int targetWidth, int targetHeight) {
		int numCols = (int)Math.round((double)targetWidth / (double)Obstacle.BODY_W);
		int numRows = (int)Math.round((double)targetHeight / (double)Obstacle.BODY_H);
		//create obstacles:
		for (int row=0;row<numRows;row++) {
			for (int col=0; col<numCols; col++) {
				Obstacle o = new Obstacle (obstacles.size(), new Vector3D (startingX+Obstacle.BODY_W*col,startingY+Obstacle.BODY_H*row,0), this);
				obstacles.add(o);
				ReportController.getSingletonObject().registerObstaclePosition(o);
			}
		}
	}
	
	public void createObstacleDiagonal(int startingX, int startingY, double steepness, int targetLength) {
		//create obstacles:
		for (int i=0;i<targetLength;i++) {
			int x = startingX+(int)(Obstacle.BODY_W*steepness)*i;
			int y = startingY+Obstacle.BODY_H*i;
			Obstacle o = new Obstacle (obstacles.size(), new Vector3D (x,y,0), this);
			obstacles.add(o);
			ReportController.getSingletonObject().registerObstaclePosition(o);	
		}
	}
	
	public void removeAllObstacles() {
		this.obstacles = null;
		this.obstacles = new ArrayList<Obstacle>();
	}
	
	//=================================================================================== GETTERS / SETTERS
	public void registerAgentPosition(Vector3D position) {
		
	}
	
	public Agent getAgent() {
		return agent;
	}
	
	public void setAgent(Agent newAgent) {
		agent = null;
		agent = newAgent;
	}
	
	public Obstacle getObstacleWithId(int id_) {
		return obstacles.get(id_);
	}
	
	public List <Obstacle> getObstacles() {
		return obstacles;
	}
	
	public void setOwner(AdaptiveNagivation owner) {
		this.owner = owner;
	}

	public AdaptiveNagivation getOwner() {
		return owner;
	}



	public void setFood(ArrayList <Food> food) {
		this.food = food;
	}

	public ArrayList <Food> getFood() {
		return food;
	}
}
