/*
 * Downloaded from lenkaspace.net
 * 
 * Paper about this project: http://lenkaspace.net/papers/iteratedPrisonersDilemmaNeuralNetworks.php
 * Email: contact@lenkaspace.net
 */

package net.lenkaspace.adaptiveNavigation.reports;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;

import javax.swing.JFrame;
import javax.swing.JPanel;

import net.lenkaspace.adaptiveNavigation.Settings;
import net.lenkaspace.adaptiveNavigation.model.Agent;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;


public class BaseReport {

	protected Dimension size;
	
	float[] temp = Color.RGBtoHSB(102, 102, 0, null);
	private Color [] cellColors = {Color.blue, Color.magenta, Color.black, Color.green,Color.cyan,Color.pink,Color.darkGray, Color.red, Color.getHSBColor(temp[0], temp[1], temp[2]) };
	
	public BaseReport(Dimension size_) {
		size = size_;	
	}
	
	public void display (String title_, String xAxisTitle_, String yAxisTitle_, XYDataset dataset_, int reportType_, int argument_) {
		JFrame frame = new JFrame(title_);	
		frame.setSize((int)size.width,(int)size.height);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setLocation((int)(Settings.WINDOW_W/2 - size.width/2),(int)(Settings.WINDOW_H/2- size.height/2));
		
		frame.setEnabled(true); 
		frame.setFocusable(true);
		frame.setResizable(false);
		
		JPanel contentPanel = new JPanel();
		contentPanel.setBackground(Color.WHITE);
				
        final JFreeChart chart = createChart(title_, xAxisTitle_, yAxisTitle_, dataset_);
        chart.getXYPlot().getDomainAxis().setLabelFont(new Font("Tahoma", Font.BOLD, 12));
        final ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new java.awt.Dimension(this.size.width, this.size.height-35));
        
        XYItemRenderer renderer = chart.getXYPlot().getRenderer();
        //----------- define colors for different report types:
        if (reportType_ == ReportController.REPORT_STATES) {
        	for (int i=0; i<Agent.NUM_CELLS; i++) {
        		int seriesNo = i*2 + 1;
        		//--- activation look:
        		renderer.setSeriesPaint(seriesNo,cellColors[i] );
        		renderer.setSeriesShape(seriesNo, new Rectangle2D.Double(-1.0, -1.0, 2.0, 2.0));
        		//--- irritation look:
        		renderer.setSeriesPaint(seriesNo+1,cellColors[i]);
        		renderer.setSeriesShape(seriesNo+1, new Ellipse2D.Double(-3.0, -3.0, 6.0,6.0));
        	}
        	
        } else if (reportType_ == ReportController.REPORT_WEIGHTS) {
        	for (int i=0; i<Agent.NUM_CELLS; i++) {
        		int seriesNo = i*2 + 1;
        		//--- left motor look:
        		renderer.setSeriesPaint(seriesNo,cellColors[i] );
        		renderer.setSeriesShape(seriesNo, new Rectangle2D.Double(-1.0, -1.0, 2.0, 2.0));
        		//--- right motor look:
        		renderer.setSeriesPaint(seriesNo+1,cellColors[i]);
        		renderer.setSeriesShape(seriesNo+1, new Ellipse2D.Double(-3.0, -3.0, 6.0,6.0));
        	}
        }
        
        //----- hide dataSeries based on argument:
        if (reportType_ == ReportController.REPORT_STATES || reportType_ == ReportController.REPORT_WEIGHTS) {
        	if (argument_ != ReportController.ALL && argument_ != ReportController.AVERAGE) {
        		//hide all:
        		for (int i=1; i<dataset_.getSeriesCount()-2; i++) {
        			renderer.setSeriesVisible(i, false);
        		}
        		//show selected:
        		renderer.setSeriesVisible(argument_*2 + 1, true);
        		renderer.setSeriesVisible(argument_*2 + 2, true);
        	} 
        }
        
        //---- max/min set to invisible:
        renderer.setSeriesVisibleInLegend(dataset_.getSeriesCount()-1,false);
        renderer.setSeriesPaint(dataset_.getSeriesCount()-1, Color.gray);
        renderer.setSeriesShape(dataset_.getSeriesCount()-1, new Rectangle2D.Double(-0.0, -0.0, 0.0, 0.0));
        
        renderer.setSeriesVisibleInLegend(dataset_.getSeriesCount()-2,false);
        renderer.setSeriesPaint(dataset_.getSeriesCount()-2, Color.gray);
        renderer.setSeriesShape(dataset_.getSeriesCount()-2, new Rectangle2D.Double(-0.0, -0.0, 0.0, 0.0));
		//--------------------
		contentPanel.add(chartPanel);
		
		frame.getContentPane().add(contentPanel, BorderLayout.CENTER);
		frame.setVisible(true);
	}
	
	
	private JFreeChart createChart(String title_, String xAxisTitle_, String yAxisTitle_, final XYDataset dataset) {
        
        // create the chart...
        final JFreeChart chart = ChartFactory.createXYLineChart(
        	title_,      // chart title
        	xAxisTitle_,                      // x axis label
        	yAxisTitle_,                      // y axis label
            dataset,                  // data
            PlotOrientation.VERTICAL,
            true,                     // include legend
            true,                     // tooltips
            false                     // urls
        );

        // NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...
        chart.setBackgroundPaint(Color.WHITE);

//        final StandardLegend legend = (StandardLegend) chart.getLegend();
  //      legend.setDisplaySeriesShapes(true);
        
        // get a reference to the plot for further customisation...
        final XYPlot plot = chart.getXYPlot();
        plot.setBackgroundPaint(Color.WHITE);
    //    plot.setAxisOffset(new Spacer(Spacer.ABSOLUTE, 5.0, 5.0, 5.0, 5.0));
        plot.setDomainGridlinePaint(Color.gray);
        plot.setRangeGridlinePaint(Color.gray);
        
        final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
        renderer.setSeriesLinesVisible(0, false);
        renderer.setSeriesShapesVisible(0, false);
        plot.setRenderer(renderer);

        // change the auto tick unit selection to integer units only...
        final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
        rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
        // OPTIONAL CUSTOMISATION COMPLETED.
                
        return chart;
        
    }
	
	
	
	
	
	
}
