/*
 * Downloaded from lenkaspace.net
 * 
 * Paper about this project: http://lenkaspace.net/papers/iteratedPrisonersDilemmaNeuralNetworks.php
 * Email: contact@lenkaspace.net
 */

package net.lenkaspace.adaptiveNavigation.reports;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;

import net.lenkaspace.adaptiveNavigation.AdaptiveNagivation;
import net.lenkaspace.adaptiveNavigation.Settings;
import net.lenkaspace.adaptiveNavigation.model.Agent;
import net.lenkaspace.adaptiveNavigation.model.Obstacle;

import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * ReportController
 * 	- registers data from models and passes it to reports for display
 */
public class ReportController {
	
	
	//---------------- CONSTANTS
	public static final int GROUNDMARK_OBSTACLE = -1;
	public static final int GROUNDMARK_FREE_SPACE = 0;
	public static final int GROUNDMARK_AGENT_ORIGIN = 1;
	public static final int GROUNDMARK_AGENT_BODY = 2;
	public static final int GROUNDMARK_AGENT_ORIGIN_BUMP = 3;
	
	public static final int REPORT_STATES = 0;
	public static final int REPORT_WEIGHTS = 1;
	public static final int REPORT_BOUNCES = 2;
	public static final int REPORT_GROUND_COVERED = 3;
	public static final int REPORT_TEXT = 4;
	public static final int REPORT_SPEED = 5;
	
	public static final int CELL_0 = 0;
	public static final int CELL_1 = 1;
	public static final int CELL_2 = 2;
	public static final int CELL_3 = 3;
	public static final int CELL_4 = 4;
	public static final int CELL_5 = 5;
	public static final int CELL_6 = 6;
	public static final int CELL_7 = 7;
	public static final int ALL = 8;
	public static final int AVERAGE = 9;
	
	//-------------------
	
	private BaseReport baseReport;
	private BaseTextReport baseTextReport;
	private static ReportController ref;
	
	private AdaptiveNagivation controller;
	
	private List<XYSeries> dataCellStates;
	private List<XYSeries> dataWeights;
	private List<XYSeries> dataBounces;
	private List<XYSeries> dataGroundCovered;
	private List<XYSeries> dataSpeed;
	
	private double tempDataBounces;
	private double tempDataGroundCovered;
	private double tempDataSpeed;
	private double [][] tempDataWeights;
	private double [][] tempDataCellStates;
	
	private int [][] groundRecord;
	
	
	private double totalAverageBumps;
	private int totalAverageBumpsSince;
	private double totalAverageSpeed;
	private int totalAverageSpeedSince;
	
	 
	
	private ReportController() 	{
		reset();
		baseReport = new BaseReport(new Dimension(Settings.WINDOW_W - 40,400));
		baseTextReport = new BaseTextReport(new Dimension(300,300));
	
		
		
	}
	
	public static ReportController getSingletonObject()	  {
	  if (ref == null)
	        // it's ok, we can call this constructor
	     ref = new ReportController();		
	  return ref;
	}
	
	public Object clone() throws CloneNotSupportedException {
	    throw new CloneNotSupportedException(); 
	}
	
	/**
	 * Reset all data
	 */
	public void reset() {
		dataCellStates = new ArrayList <XYSeries>();
		dataWeights = new ArrayList <XYSeries>();
		dataBounces = new ArrayList <XYSeries>();
		dataGroundCovered = new ArrayList <XYSeries>();
		dataSpeed = new ArrayList <XYSeries>();
		
		tempDataBounces = 0.0;
		tempDataSpeed = 0.0;
		tempDataWeights = new double [Agent.NUM_CELLS][2];
		tempDataCellStates = new double [Agent.NUM_CELLS][2];
		tempDataGroundCovered = 0.0;
		
		groundRecord = new int[Settings.WORLD_W][Settings.WORLD_H];
		
		this.resetAverages();
		
	}
	
	//=================================== showing reports:
	/**
	 * Resolve report type requested from UI and show appropriate report
	 */
	public void showReport(int reportType_, int argument_) {
		switch(reportType_) {
			case REPORT_STATES:
				baseReport.display("Cell state variables ", "Time units", "Value", this.prepareXYSeriesForDisplay(this.dataCellStates), reportType_, argument_);
				break;
			case REPORT_WEIGHTS:
				baseReport.display("Connection strengths to left (L) and right (R) motors", "Time units", "Connection strength", this.prepareXYSeriesForDisplay(this.dataWeights), reportType_, argument_);
				break;
			case REPORT_BOUNCES:
				baseReport.display("Bounces (% of time colliding out of each second)", "Time units", "Bounce %", this.prepareXYSeriesForDisplay(this.dataBounces), reportType_, argument_);
				break;
			
			case REPORT_GROUND_COVERED:
				baseReport.display("Ground covered (% of free space)", "Time units", "Ground covered %", this.prepareXYSeriesForDisplay(this.dataGroundCovered), reportType_, argument_);
				break;
			case REPORT_SPEED:
				baseReport.display("Speed", "Time units", "Speed", this.prepareXYSeriesForDisplay(this.dataSpeed), reportType_, argument_);
				break;
			case REPORT_TEXT:
				baseTextReport.display("STATS", this);
				break;
		}
		
		
	
	}
	
	
	//======================================= registering data:
	
	/**
	 * Register cell activation and irritation
	 */
	//------------ cell state
	public void registerCellState(int cellId, double activation, double irritation, double output) {
		//-- add to temp counter:
		tempDataCellStates[cellId][0] += activation;
		tempDataCellStates[cellId][1] += irritation;
		
		//create collection:
		if (dataCellStates.size() == 0) {
			//String[] varNames = {"activation", "irritation", "output", "[max]"};
			String[] varNames = {"0A", "0IR","1A", "1IR","2A", "2IR","3A", "3IR","4A", "4IR","5A", "5IR","6A", "6IR", "7A", "7IR", "8A", "8IR","[max]", "[min]"};
			for (int i=0; i<varNames.length; i++) {
				XYSeries series = new XYSeries(varNames[i]);
				series.setDescription(varNames[i]);
				dataCellStates.add(series);
			}	
		}
		
		//add initial values:
		if (dataCellStates.get(cellId*2).isEmpty()) {
			//add additional values:
			dataCellStates.get(cellId*2).add(controller.getTime(),activation);
			dataCellStates.get(cellId*2+1).add(controller.getTime(),irritation);
			
			if (cellId == 0) {
				dataCellStates.get((Agent.NUM_CELLS)*2).add(controller.getTime(),1);
				dataCellStates.get((Agent.NUM_CELLS)*2 +1).add(controller.getTime(),0);
			}
		}
		
		//----
		if (this.isWritable()) {
			
			double averActivation = tempDataCellStates[cellId][0] / (double) Settings.TIME_UNIT_INTERVAL;
			double averIrritation = tempDataCellStates[cellId][1] / (double) Settings.TIME_UNIT_INTERVAL;
			
			dataCellStates.get(cellId*2).add(controller.getTime(),averActivation);
			dataCellStates.get(cellId*2+1).add(controller.getTime(),averIrritation);
			if (cellId == 0) {
				dataCellStates.get((Agent.NUM_CELLS)*2).add(controller.getTime(),1);
				dataCellStates.get((Agent.NUM_CELLS)*2+1).add(controller.getTime(),0);
			}
			
			//-- reset temp counter:
			tempDataCellStates[cellId][0] = 0.0;
			tempDataCellStates[cellId][1] = 0.0;
			
			
			
		}
	}
	
	//--------------- weights
	/**
	 * Register cell connection weights
	 */
	public void registerWeights(int cellId, double[] weights) {
		//-- add to temp counter:
		tempDataWeights[cellId][0] += weights[0];
		tempDataWeights[cellId][1] += weights[1];
		
		//create collection:
		if (dataWeights.size() == 0) {
			String[] weightNames = {"0L", "0R", "1L", "1R", "2L", "2R", "3L", "3R", "4L", "4R", "5L", "5R", "6L", "6R", "7L", "7R", "8L","8R", "[max]","[min]"};
			for (int i=0; i<weightNames.length; i++) {
				XYSeries series = new XYSeries(weightNames[i]);
				series.setDescription(weightNames[i]);
				dataWeights.add(series);
			}	
			
			
		}
		
		//add initial values:
		if (dataWeights.get(cellId*2).isEmpty()) {
			dataWeights.get(cellId*2).add(controller.getTime(), weights[0]);
			dataWeights.get(cellId*2+1).add(controller.getTime(), weights[1]);
			
			if (cellId == 0) {
				dataWeights.get((Agent.NUM_CELLS)*2).add(controller.getTime(),1);
				dataWeights.get((Agent.NUM_CELLS)*2 + 1).add(controller.getTime(),-1);
			}
		}
		
		if (this.isWritable()) {
			
			
			double averLeft = tempDataWeights[cellId][0] / (double) Settings.TIME_UNIT_INTERVAL;
			double averRight = tempDataWeights[cellId][1] / (double) Settings.TIME_UNIT_INTERVAL;
			dataWeights.get(cellId*2).add(controller.getTime(), averLeft);
			dataWeights.get(cellId*2+1).add(controller.getTime(), averRight);
			
			if (cellId == 0) {
				dataWeights.get((Agent.NUM_CELLS)*2).add(controller.getTime(),1);
				dataWeights.get((Agent.NUM_CELLS)*2 + 1).add(controller.getTime(),-1);
			}
			
			//-- reset temp counter:
			tempDataWeights[cellId][0] = 0.0;
			tempDataWeights[cellId][1] = 0.0;
			
		}	
	}
	
	//-------------- bump to objects
	/**
	 * Register hitting of obstacles, where val=0 if not hitting, 1 if hitting
	 */
	public void registerBumpToObstacle(int val, Agent agent) {
		//add to temporary value:
		tempDataBounces += val;
		
		//create collection:
		if (dataBounces.size() == 0) {
			XYSeries series = new XYSeries("Bounces");
			series.setDescription("Bounces");
			dataBounces.add(series);
			
			XYSeries series2 = new XYSeries("Max");
			series2.setDescription("Max");
			dataBounces.add(series2);
			
			XYSeries series3 = new XYSeries("Min");
			series3.setDescription("Min");
			dataBounces.add(series3);
		}
		
		//add initial values:
		if (dataBounces.get(0).isEmpty()) {
			dataBounces.get(0).add(controller.getTime(),val);
			dataBounces.get(1).add(controller.getTime(),1);
			dataBounces.get(2).add(controller.getTime(),0);
		}
		
		
		if (this.isWritable()) {
			//calculate average out of temporary value:
			double averageVal =  tempDataBounces/(double)Settings.TIME_UNIT_INTERVAL;
			dataBounces.get(0).add(controller.getTime(),averageVal);
			dataBounces.get(1).add(controller.getTime(),1);
			dataBounces.get(2).add(controller.getTime(),0);
			//reset temporary value:
			tempDataBounces = 0.0;
			//add to total average value
			this.totalAverageBumps += averageVal;
			
		}
		
		//register position of bumping:
		if (val == 1) {
			groundRecord[(int) agent.getWorldPosition().x][(int) agent.getWorldPosition().y] = ReportController.GROUNDMARK_AGENT_ORIGIN_BUMP;
		}
	}
	
	
	//-------- ground covered
	/**
	 * Register amoung of covered ground
	 */
	public void registerGroundCovered(double val) {
		//add to temporary value:
		tempDataGroundCovered += val;
		
		//create collection:
		if (dataGroundCovered.size() == 0) {
			String[] varNames = {"Value", "[max]", "[min]"};
			for (int i=0; i<varNames.length; i++) {
				XYSeries series = new XYSeries(varNames[i]);
				series.setDescription(varNames[i]);
				dataGroundCovered.add(series);
			}	
		}
		
		//add initial values:
		if (dataGroundCovered.get(0).isEmpty()) {
			dataGroundCovered.get(0).add(controller.getTime(),val);
			dataGroundCovered.get(1).add(controller.getTime(),1);
			dataGroundCovered.get(2).add(controller.getTime(),0);
		}
		
		if (this.isWritable()) {
			
			//calculate average out of temporary value:
			double averageVal =  tempDataGroundCovered/(double)Settings.TIME_UNIT_INTERVAL;
			dataGroundCovered.get(0).add(controller.getTime(),averageVal);
			dataGroundCovered.get(1).add(controller.getTime(),1);
			dataGroundCovered.get(2).add(controller.getTime(),0);
			//reset temporary value:
			tempDataGroundCovered = 0.0;
			
		}
	}
	
	//-------- speed
	/**
	 * Register agent's speed
	 */
	public void registerAgentSpeed(double val) {
		//add to temporary value:
		tempDataSpeed += val;
		
		//create collection:
		if (dataSpeed.size() == 0) {
			String[] varNames = {"Value", "[max]", "[min]"};
			for (int i=0; i<varNames.length; i++) {
				XYSeries series = new XYSeries(varNames[i]);
				series.setDescription(varNames[i]);
				dataSpeed.add(series);
			}	
		}
		
		//add initial values:
		if (dataSpeed.get(0).isEmpty()) {
			dataSpeed.get(0).add(controller.getTime(),val);
			dataSpeed.get(1).add(controller.getTime(),2);
			dataSpeed.get(2).add(controller.getTime(),0);
		}
		
		if (this.isWritable()) {
			
			//calculate average out of temporary value:
			double averageVal =  tempDataSpeed/(double)Settings.TIME_UNIT_INTERVAL;
			dataSpeed.get(0).add(controller.getTime(),averageVal);
			dataSpeed.get(1).add(controller.getTime(),2);
			dataSpeed.get(2).add(controller.getTime(),0);
			
			//reset temporary value:
			tempDataSpeed = 0.0;
			this.totalAverageSpeed += averageVal;
			
		}
	}
	
	//-------------- positions
	/**
	 * Register pixels covered by obstacles
	 */
	public void registerObstaclePosition(Obstacle obstacle) {
		//register space as occupied by obstacle:
		int startX = (int)obstacle.getWorldPosition().x - (int)Math.floor(obstacle.getSize().x/2.0);
		int startY = (int)obstacle.getWorldPosition().y - (int)Math.floor(obstacle.getSize().y/2.0);
		for (int i= startX; i<startX+obstacle.getSize().x; i++) {
			if (i>=0 && i<Settings.WORLD_W) {
				for (int j = startY; j<startY+obstacle.getSize().y; j++) {
					if (j>=0 && j < Settings.WORLD_H) {
						groundRecord[i][j] = ReportController.GROUNDMARK_OBSTACLE;
					}	
				}
			}
		}
	}
	
	/**
	 * Register pixels covered by agent's body, used when drawing path or calculating covered ground
	 */
	public void registerAgentPosition(Agent agent) {
		int startX = (int)agent.getWorldPosition().x - (int)Math.floor(agent.getSize().x/2.0);
		int startY = (int)agent.getWorldPosition().y - (int)Math.floor(agent.getSize().y/2.0);
		for (int i= startX; i<startX+agent.getSize().x; i++) {
			if (i>=0 && i<Settings.WORLD_W) {
				for (int j = startY; j<startY+agent.getSize().y; j++) {
					if (j>=0 && j < Settings.WORLD_H) {
						if (i == (int)agent.getWorldPosition().x && j == (int)agent.getWorldPosition().y) {
							if (groundRecord[i][j] != GROUNDMARK_AGENT_ORIGIN_BUMP) {
								groundRecord[i][j] = ReportController.GROUNDMARK_AGENT_ORIGIN;
							}
						} else {
							if (groundRecord[i][j] != ReportController.GROUNDMARK_AGENT_ORIGIN && groundRecord[i][j] != GROUNDMARK_AGENT_ORIGIN_BUMP ) {
								groundRecord[i][j] = ReportController.GROUNDMARK_AGENT_BODY;
							}
						}
						
					}	
				}
			}
		}
		//register this value
		this.registerGroundCovered(this.getGroundCovered()[1]);
	}
	
	/**
	 * Set all registered agent position pixels as free
	 */
	public void clearAgentPositions() {
		for (int i=0; i<Settings.WORLD_W; i++) {
			for (int j=0; j< Settings.WORLD_H; j++) {
				if (groundRecord[i][j] == ReportController.GROUNDMARK_AGENT_ORIGIN || groundRecord[i][j] == ReportController.GROUNDMARK_AGENT_BODY || groundRecord[i][j] == GROUNDMARK_AGENT_ORIGIN_BUMP) {
					groundRecord[i][j] = ReportController.GROUNDMARK_FREE_SPACE;
				}
			}
		}
	}
	
	

	//----------------------------------------
	/**
	 * Convert data for display in a line graph
	 */
	private XYDataset prepareXYSeriesForDisplay(List<XYSeries> rawData) {
		
		XYSeriesCollection dataset = new XYSeriesCollection();
		dataset.addSeries(new XYSeries("Legend: ")); //needs a series at the beginning, otherwise won't display...
		
		//go through the list of rawData (e.g. multiple variables the report should show:
		for (int var=0; var < rawData.size(); var++) {
			XYSeries rawSeries = rawData.get(var);
			XYSeries displaySeries = new XYSeries(rawSeries.getDescription());
			int dataSize = rawSeries.getItemCount();
			int desiredDataSize = 100;
			
			if (dataSize <= desiredDataSize) {
				try {
					displaySeries = (XYSeries) rawSeries.clone();
				} catch (CloneNotSupportedException e) {
					e.printStackTrace();
				}
			} else {
				for (int i = 0; i<desiredDataSize-1; i++) {
					//make sure it marks every Nth datapoint of original data, so that total num of X-axis points is dataSize
					int k = (int)Math.ceil((double)dataSize/(double)(desiredDataSize-1))*(i);
					if (k<dataSize) {
						displaySeries.add(k,rawSeries.getY(k));
					}			
				}
				//----set last value of graph's data set to be the last value of the original data:
				displaySeries.add(dataSize, rawSeries.getY(dataSize-1));
				//-----
			} 
			
			dataset.addSeries(displaySeries);  
		
		}
			        
	    return dataset;
	}
	
	//=================================================== OTHER
	
	public void showAllReports() {
		this.showReport(REPORT_STATES, ALL);
		this.showReport(REPORT_WEIGHTS, ALL);
		this.showReport(REPORT_GROUND_COVERED, -1);
		this.showReport(REPORT_SPEED, -1);
		this.showReport(REPORT_BOUNCES, -1);
		this.showReport(REPORT_TEXT, -1);
	}

	//=========================================================== GETTERS / SETTERS
	/**
	 * Calculate percentage of space not occupied by obstacles
	 */
	public int getFreeSpace() {
		int freeSpaceCounter = 0;
		for (int i=0; i<Settings.WORLD_W; i++) {
			for (int j=0; j<Settings.WORLD_H; j++) {
				if (groundRecord[i][j] != ReportController.GROUNDMARK_OBSTACLE) {
					freeSpaceCounter++;
				}
			}
		}
		return freeSpaceCounter;
	}
	
	/**
	 * Return space covered by agent as array [coveredSpaceInPixels, coveredSpacePercentageOfFreeSpace]
	 */
	public double[] getGroundCovered() {
		int coveredSpaceCounter = 0;
		int freeSpaceCounter = 0;
		for (int i=0; i<Settings.WORLD_W; i++) {
			for (int j=0; j<Settings.WORLD_H; j++) {
				if (groundRecord[i][j] == ReportController.GROUNDMARK_AGENT_ORIGIN || groundRecord[i][j] == ReportController.GROUNDMARK_AGENT_BODY || groundRecord[i][j] == GROUNDMARK_AGENT_ORIGIN_BUMP) {
					coveredSpaceCounter++;
				}
				if (groundRecord[i][j] != ReportController.GROUNDMARK_OBSTACLE) {
					freeSpaceCounter++;
				}
			}
		}
		double percent = (double)coveredSpaceCounter / (double) freeSpaceCounter;
		return new double[] {coveredSpaceCounter,percent, freeSpaceCounter};
	}
	
	public int[][] getGroundRecord() {
		return groundRecord;
	}
	
	
	public double getAverageBumps() {
		return this.totalAverageBumps / (double)(this.controller.getTime() - this.totalAverageBumpsSince);
	}
	
	public double getAverageSpeed() {
		return this.totalAverageSpeed / (double)(this.controller.getTime() - this.totalAverageSpeedSince);
	}
	
	public void resetAverages() {
		this.totalAverageBumps = 0;
		this.totalAverageSpeed = 0;
		if (this.controller != null) {
			this.totalAverageBumpsSince = this.controller.getTime();
			this.totalAverageSpeedSince = this.controller.getTime();
		}
	}
	
	public double getTotalAverageBumps() {
		return this.totalAverageBumps;
	}
	
	public int getTotalAverageBumpsSince() {
		return this.totalAverageBumpsSince;
	}
	
	public boolean isWritable() {
		return(controller.getTimeCounter() == Settings.TIME_UNIT_INTERVAL);
	}
	public void setController(AdaptiveNagivation controller) {
		this.controller = controller;
	}

	public AdaptiveNagivation getController() {
		return controller;
	}

	

	
	
	

	
	

	
}
